
#include <stdio.h>
#include <stdlib.h>
#include "cell.h"
#include "rif_rgb.h"
#include "sif.h"
#include "sif_rgb.h"

extern void	Error();

static int	Nx;
static double	Bx;

static void	Add(sR,sG,sB,Ry,cR,cG,cB)
double		*sR,*sG,*sB,Ry;
Cell		*cR,*cG,*cB;
{
	int	x,h=(-1),next=1;
	double	Rx,lR=0.0,lG=0.0,lB=0.0;

	for (Rx=0.0, x=0; x<Nx; x++, Rx-=1.0) {
	    if (next) {
		++h; sR[h]+=(-Rx*lR); sG[h]+=(-Rx*lG); sB[h]+=(-Rx*lB); Rx+=Bx;
	    }
	    lR=Ry*(double)cR[x]; lG=Ry*(double)cG[x]; lB=Ry*(double)cB[x];
	    if (next=Rx<=1.0) {
		sR[h]+=(Rx*lR); sG[h]+=(Rx*lG); sB[h]+=(Rx*lB);
	    }
	    else {
		sR[h]+=lR; sG[h]+=lG; sB[h]+=lB;
	    }
	}
}

int	main(argc,argv)
int	argc;
char	**argv;
{
	int	Ny,BPS,Nh,H,Nv,V,v,h,y,next,gray=1;
	Cell	**cR,**cG,**cB,*lR,*lG,*lB,r,g,b;
	double	By,**sR,**sG,**sB,Ry,ratio,factor,Ax,Ay;
	char	*bbs="bad block size.";

	if (argc!=4 && argc!=5) Error("usage : pnr_RGB orgTIFF Bxy newTIFF\n"
				      "        pnr_RGB orgTIFF Bx By newTIFF");

	ReadImageFile_RGB(argv[1],&Nx,&Ny,&BPS,&cR,&cG,&cB,NULL);

	if (argc==4) {
	    if ((Bx=By=atof(argv[2]))<1.0) Error(bbs);
	}
	else
	    if ((Bx=atof(argv[2]))<1.0 ||
		(By=atof(argv[3]))<1.0) Error(bbs);

	Nh=(H=(int)((double)(Nx-1)/Bx))+1;
	Nv=(V=(int)((double)(Ny-1)/By))+1;

	if ((sR=(double **)malloc(sizeof(double *)*(size_t)Nv))==NULL ||
	    (sG=(double **)malloc(sizeof(double *)*(size_t)Nv))==NULL ||
	    (sB=(double **)malloc(sizeof(double *)*(size_t)Nv))==NULL ||
	(*sR=(double *)malloc(sizeof(double)*(size_t)Nv*(size_t)Nh))==NULL ||
	(*sG=(double *)malloc(sizeof(double)*(size_t)Nv*(size_t)Nh))==NULL ||
	(*sB=(double *)malloc(sizeof(double)*(size_t)Nv*(size_t)Nh))==NULL)
	    Error("no allocatable memory for sum.");

	for (v=0; v<Nv; v++) {
	    if (v!=0) {
		sR[v]=sR[v-1]+Nh; sG[v]=sG[v-1]+Nh; sB[v]=sB[v-1]+Nh;
	    }
	    for (h=0; h<Nh; h++) sR[v][h]=sG[v][h]=sB[v][h]=0.0;
	}
	v=(-1); lR=cR[0]; lG=cG[0]; lB=cB[0]; next=1;
	for (Ry=0.0, y=0; y<Ny; y++, Ry-=1.0) {
	    if (next) {
		++v; Add(sR[v],sG[v],sB[v],-Ry,lR,lG,lB); Ry+=By;
            }
	Add(sR[v],sG[v],sB[v],(next=Ry<=1.0)?Ry:1.0,lR=cR[y],lG=cG[y],lB=cB[y]);
        }
	ratio=255.0/(double)((1<<BPS)-1);

#define RGB(h,v)	\
	r=cR[v][h]=(Cell)(sR[v][h]*factor+0.5); \
	g=cG[v][h]=(Cell)(sG[v][h]*factor+0.5); \
	b=cB[v][h]=(Cell)(sB[v][h]*factor+0.5); if (gray) gray=r==g && r==b

	factor=ratio/Bx/By;
	for (v=0; v<V; v++)
	for (h=0; h<H; h++) {
	    RGB(h,v);
	}
	factor=ratio/(Ax=(double)Nx-Bx*(double)H)/By;
	for (v=0; v<V; v++) {
	    RGB(H,v);
	}
	factor=ratio/Bx/(Ay=(double)Ny-By*(double)V);
	for (h=0; h<H; h++) {
	    RGB(h,V);
	}
	factor=ratio/Ax/Ay; RGB(H,V);

	if (gray)
	    StoreImageFile(argv[argc-1],Nh,Nv,8,cR,NULL);
	else
	    StoreImageFile_RGB(argv[argc-1],Nh,Nv,8,cR,cG,cB,NULL);

	return 0;
}
